/*#############################################################
File Name       : ex11_led_count_on_16bit_port.c
Author          : Grant Phillips
Date Modified   : 06/09/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F0-Discovery

Description     : The same as example ex7_led_count.c, but 
									demonstrates how to write to 8 bits on a 
									16-bit port without influencing the state of
									the other 8 bits.	       

Requirements    : * STM32F0-Discovery Board
              
Circuit         : * 8 LEDs connected to PC7 - PC0
									* You can also connect 8 LEDs to PC15 - PC8 to
									  prove that this value will not change while
										the system counts on PC7 - PC0

See the STM32F0-Discovery User Manual (UM1525) for the block 
diagram of the STM32F051R8T6 processor (p13), a summary of
the GPIO connections (p20-26) and the schematic diagram (p37-39)

##############################################################*/

/* #includes -------------------------------------------------*/
#include "main.h"																								//main library to include for device drivers, peripheral drivers, etc.

/* #defines --------------------------------------------------*/

/* #function prototypes --------------------------------------*/

/* #global variables -----------------------------------------*/



// Unused global variables that have to be included to ensure correct compiling */
// ###### DO NOT CHANGE ######
// ===============================================================================
__IO uint32_t TimingDelay = 0;																	//used with the Delay function
__IO uint8_t DataReady = 0;
__IO uint32_t USBConnectTimeOut = 100;
__IO uint32_t UserButtonPressed = 0;
__IO uint8_t PrevXferComplete = 1;
// ===============================================================================



int main(void)
{
	GPIO_InitTypeDef        GPIO_InitStructure;										//structure used for setting up a GPIO port
	RCC_ClocksTypeDef 			RCC_Clocks;														//structure used for setting up the SysTick Interrupt
	
	unsigned int count=0;            	//integer variable to store count
	unsigned int temp;								//temporary variable to store the state of GPIOC
	
	/* GPIOC Periph clock enable */
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);
  
  /* Configure PC15 - PC0 in output pushpull mode */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_All;				//in this example ALL the pins (PC15 - PC0) are outputs
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;									
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;								
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;							
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;							
  GPIO_Init(GPIOC, &GPIO_InitStructure);												
  	
	/* Set the SysTick Interrupt to occur every 1ms) */
	RCC_GetClocksFreq(&RCC_Clocks);
	if (SysTick_Config(RCC_Clocks.HCLK_Frequency / 1000))
		while(1);																										
	
	/* To demonstrate the fact the changing of PC7 - PC0 will not change the state
	of PC15 - PC8, we will write a dummy value to PC15 to PC8 which should stay
	constant as the count increments on the LEDs on PC7 - PC0 */
	GPIO_Write(GPIOC, 0xaa00);			//16bit value; PC15 - PC8 = 0xaa; PC7 - PC0 = 0x00
	
	/* Main program loop */
  while (1)
  {
		temp = GPIO_ReadOutputData(GPIOC);	//read the current output state of GPIOC and store as a 16bit value
		temp = temp & 0xff00;								//PC15 - PC8 gets bitwise ANDed with 1's (0xff), which basically means
																				//that whatever they were, they will stay (1 AND 1 = 1; 0 AND 1 = 0)
																				//PC7 - PC0 gets bitwise ANDed with 0's, which means that those bits will
																				//all be changed to 0's (1 AND 0 = 0; 0 AND 0 = 0)
		temp = temp | count;								//count is a 16bit value, but in this example only the lower 8bits are used, 
																				//so when the count is 255, the value can be seen as 0x00ff.  So this value 
																				//now gets bitwise ORed with the count (e.g. 255 or 0x00ff), which means that
																				//PC15 - PC8 stays the same (1 OR 0 = 1; 0 OR 0 = 0), and PC7 - PC0 is changed
																				//from all 0's to the count value (e.g. 255 or ff) (0 OR 1 = 1; 0 OR 0 = 0)
    GPIO_Write(GPIOC, temp);     			  //display the count value on GPIOC, but keep PC15-PC8 the same
		
		/* the above 4 lines could also have been replaced by 1 line:
		GPIO_Write(GPIOC, (GPIO_ReadOutputData(GPIOC) & 0xff00) | count);
		*/
		
    count = count + 1;            			//increment count
    if(count == 256)
       count = 0;                 			//reset counter if it reaches 256 *** see writeup below
    Delay(200);            							//delay 200ms													
  }	
}

/*
count was delcared as an unsigned integer variable, which means it is a 
16-bit variable with no sign bit and can therefore hold a value of 0 to
65535.  

If it was declared as
int count;
it would still be a 16-bit variable, but with a sign bit and can therefore
hold a value of -32768 to +32767.

To create a 8-bit variable (which uses less memory), we can declare it as
char count;            (signed 8-bit variable; -128 to 127)
unsigned char count;   (unsigned 8-bit variable; 0 to 255)

So if count was delcared as 
unsigned char count;
then the 
if(count == 256)
statement was not necessary, because the counter would have simply "rolled
over" from 255 to 0 seeing that it can only store 8 bits.
*/


// -------------------------------------------------------------------------------

// Function to insert a timing delay of nTime
// ###### DO NOT CHANGE ######
void Delay(__IO uint32_t nTime)
{
  TimingDelay = nTime;

  while(TimingDelay != 0);
}

// Function to Decrement the TimingDelay variable.
// ###### DO NOT CHANGE ######
void TimingDelay_Decrement(void)
{
  if (TimingDelay != 0x00)
  { 
    TimingDelay--;
  }
}

// Unused functions that have to be included to ensure correct compiling
// ###### DO NOT CHANGE ######
// =======================================================================
uint32_t L3GD20_TIMEOUT_UserCallback(void)
{
  return 0;
}

uint32_t LSM303DLHC_TIMEOUT_UserCallback(void)
{
  return 0;
}
// =======================================================================



